'
' ####################
' #####  PROLOG  #####
' ####################
'
' A game of life simulation.
'
' v0.0002 - added rule where newborn has parent's color
'         - added new functions InitializeParentColor and PutTextAttributes
'         - added .cParent to CELL type
'
VERSION "0.0002"
CONSOLE
'
	IMPORT "xst"
	IMPORT "xsx"
	IMPORT "xio"

'	IMPORT "xst_s.lib"
' IMPORT "xsx_s.lib"
' IMPORT "xio_s.lib"

	IMPORT "kernel32"

TYPE CELL
	XLONG .state  		' on = $$TRUE, off = $$FALSE
	XLONG .color			' color
	XLONG .con				' character on
	XLONG .coff				' character off
	XLONG .generation	' current generation count since birth
	XLONG .neighbors  ' current count of live neigbors
	XLONG .cParent    ' color of parent cell (has 3 neighbors alive)
END TYPE

'
DECLARE FUNCTION Entry ()
DECLARE FUNCTION InitializeState (CELL cell[])
DECLARE FUNCTION SetCellCharOn (x, y, char, fAll, CELL cell[])
DECLARE FUNCTION SetCellCharOff (x, y, char, fAll, CELL cell[])
DECLARE FUNCTION GetNeighbors (x, y, CELL cell[])
DECLARE FUNCTION GetAllNeighbors (CELL cell[])
DECLARE FUNCTION LiveOrDie (CELL cell[], fColor)
DECLARE FUNCTION DisplayState (CELL cell[], fColor)
DECLARE FUNCTION InitializeParentColor (CELL cell[])

' console functions
DECLARE FUNCTION PutTextAttributes (hStdOut, attribute, length, x, y)
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	CELL cell[]

	DIM cell[79, 24]
	
	XioSetCursorType (XioGetStdOut(), 0)					' turn off cursor
	SetCellCharOn (0, 0, 'O', $$TRUE, @cell[])		' set cell alive character
	SetCellCharOff (0, 0, ' ', $$TRUE, @cell[])		' set cell dead character
	InitializeState (@cell[])											' set a random initial state 
	InitializeParentColor (@cell[])								' init parent color

	DO
		GetAllNeighbors (@cell[])										' count live neighbors
		LiveOrDie (@cell[], $$TRUE)									' does cell live or die?
		DisplayState (@cell[], $$TRUE)							' display current generation
		XstSleep (70)																' slow it down
	LOOP UNTIL XioInkey () 

	
'  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION

FUNCTION InitializeState (CELL cell[])

	IFZ cell[] THEN RETURN ($$TRUE)

	' create seeded starting point
	' randomly set state to on or off
	uppX = UBOUND (cell[])
	uppY = UBOUND (cell[uppX,])
	FOR x = 0 TO uppX
		FOR y = 0 TO uppY
			cell[x,y].state = XstRandomRange (0, 1)
		NEXT y
	NEXT x

END FUNCTION

FUNCTION SetCellCharOn (x, y, char, fAll, CELL cell[])

	IF char < 32 THEN RETURN ($$TRUE)
	IFZ cell[] THEN RETURN ($$TRUE)
			
	uppX = UBOUND (cell[])
	uppY = UBOUND (cell[uppX,])
	
	IF x > uppX THEN RETURN ($$TRUE)
	IF y > uppY THEN RETURN ($$TRUE)
		
	IF fAll THEN
		FOR x = 0 TO uppX
			FOR y = 0 TO uppY
				cell[x,y].con = char
			NEXT y
		NEXT x
	ELSE
		cell[x,y].con = char
	END IF

END FUNCTION

FUNCTION SetCellCharOff (x, y, char, fAll, CELL cell[])

	IF char < 32 THEN RETURN ($$TRUE)
	IFZ cell[] THEN RETURN ($$TRUE)
			
	uppX = UBOUND (cell[])
	uppY = UBOUND (cell[uppX,])
	
	IF x > uppX THEN RETURN ($$TRUE)
	IF y > uppY THEN RETURN ($$TRUE)
		
	IF fAll THEN
		FOR x = 0 TO uppX
			FOR y = 0 TO uppY
				cell[x,y].coff = char
			NEXT y
		NEXT x
	ELSE
		cell[x,y].coff = char
	END IF

END FUNCTION

FUNCTION GetNeighbors (x, y, CELL cell[])

' return count of live neighbors and parent color if no of neighbors = 3

	IFZ cell[] THEN RETURN ($$TRUE)
			
	uppX = UBOUND (cell[])
	uppY = UBOUND (cell[uppX,])
	
	IF x > uppX THEN RETURN ($$TRUE)
	IF y > uppY THEN RETURN ($$TRUE)
	
	' eight neighbors
	xm1 = x - 1
	xp1 = x + 1
	ym1 = y - 1
	yp1 = y + 1
	
	' get count of neighbors and capture neighbor colors
	
	IF (xm1 >= 0) && (ym1 >= 0) THEN 
		IF cell[xm1, ym1].state THEN INC count : cParent = cell[xm1, ym1].color
	END IF
	
	IF (ym1 >= 0) THEN
		IF cell[x  , ym1].state THEN INC count : cParent = cell[x  , ym1].color
	END IF
	
	IF (xp1 <= uppX) && (ym1 >= 0) THEN
		IF cell[xp1, ym1].state THEN INC count : cParent = cell[xp1, ym1].color
	END IF
	
	IF (xm1 >= 0) THEN
		IF cell[xm1,   y].state THEN INC count : cParent = cell[xm1,   y].color
	END IF
	
	IF xp1 <= uppX THEN
		IF cell[xp1,   y].state THEN INC count : cParent = cell[xp1,   y].color
	END IF
	
	IF (xm1 >= 0) && (yp1 <= uppY) THEN
		IF cell[xm1, yp1].state THEN INC count : cParent = cell[xm1, yp1].color
	END IF
	
	IF (yp1 <= uppY) THEN
		IF cell[x  , yp1].state THEN INC count : cParent = cell[x  , yp1].color
	END IF
	
	IF (xp1 <= uppX) && (yp1 <= uppY) THEN
		IF cell[xp1, yp1].state THEN INC count : cParent = cell[xp1, yp1].color
	END IF
	
	' if no of neighbors is 3, then save parent color
	IF count = 3 THEN 
		cell[x,y].cParent = cParent
	ELSE
		cell[x,y].cParent = $$BLACK
	END IF

	RETURN count

END FUNCTION

FUNCTION GetAllNeighbors (CELL cell[])

' get all live neighbors

	IFZ cell[] THEN RETURN ($$TRUE)
			
	uppX = UBOUND (cell[])
	uppY = UBOUND (cell[uppX,])
	
	FOR x = 0 TO uppX
		FOR y = 0 TO uppY
			cell[x,y].neighbors = GetNeighbors (x, y, @cell[])
		NEXT y
	NEXT x

END FUNCTION

FUNCTION LiveOrDie (CELL cell[], fColor)

' rules:
'	1. Any live cell with fewer than two live neighbours dies, as if by loneliness.
'	2. Any live cell with more than three live neighbours dies, as if by overcrowding.
'	3. Any live cell with two or three live neighbours lives, unchanged, to the next generation.
'	4. Any dead cell with exactly three live neighbours comes to life.
' 5. Newborn cell keeps parent color

	IFZ cell[] THEN RETURN ($$TRUE)
			
	uppX = UBOUND (cell[])
	uppY = UBOUND (cell[uppX,])
	
	FOR x = 0 TO uppX
		FOR y = 0 TO uppY
			IF cell[x,y].state THEN
				IF cell[x,y].neighbors < 2 THEN  						' see rule 1 
					cell[x,y].state = $$FALSE 
					IF fColor THEN cell[x,y].color = $$BLACK
				END IF
				IF cell[x,y].neighbors > 3 THEN 						' see rule 2
					cell[x,y].state = $$FALSE 
					IF fColor THEN cell[x,y].color = $$BLACK
				END IF
			ELSE
				IF cell[x,y].neighbors = 3 THEN  						' see rule 4 
					IF fColor THEN 
						cell[x,y].color = cell[x,y].cParent 	 	' see rule 5
					END IF
					cell[x,y].state = $$TRUE 
				END IF
			END IF
		NEXT y
	NEXT x

END FUNCTION

FUNCTION DisplayState (CELL cell[], fColor)

	IFZ cell[] THEN RETURN ($$TRUE)
			
	uppX = UBOUND (cell[])
	uppY = UBOUND (cell[uppX,])
	
	hStdOut = XioGetStdOut ()

	FOR x = 0 TO uppX
		FOR y = 0 TO uppY
			IF cell[x,y].state THEN 
				text$ = CHR$(cell[x,y].con)
			ELSE
				text$ = CHR$(cell[x,y].coff)
			END IF
			IF fColor THEN
				color = cell[x,y].color
				PutTextAttributes (hStdOut, color, LEN(text$), x, y)
'				XioPutConsoleTextAttributes (hStdOut, color, LEN(text$), x, y)
			END IF 
			XioPutConsoleText (hStdOut, text$, x, y)
		NEXT y
	NEXT x

END FUNCTION

FUNCTION InitializeParentColor (CELL cell[])

	IFZ cell[] THEN RETURN ($$TRUE)

	' randomly set color for alive cells
	uppX = UBOUND (cell[])
	uppY = UBOUND (cell[uppX,])
	FOR x = 0 TO uppX
		FOR y = 0 TO uppY
			IF cell[x,y].state THEN									' if alive, then randomly assign parent color
				IF XstRandomRange (0, 1) THEN					' get 1 or 0
					cell[x,y].color = $$LIGHTMAGENTA 		' text is magenta
				ELSE
					cell[x,y].color = $$LIGHTRED				' text is red 
				END IF
			ELSE
				cell[x,y].color =  $$WHITE						' text is white 
			END IF
		NEXT y
	NEXT x

END FUNCTION

FUNCTION PutTextAttributes (hStdOut, attribute, length, x, y)

	writeCoord = MAKELONG (x, y)
	IFZ WriteConsoleOutputAttribute (hStdOut, &attribute, length, writeCoord, &attrsWritten) THEN GOSUB Error

' ***** Error *****
SUB Error
	errno = GetLastError ()
	XstSystemErrorToError (errno, @error)
	lastErr = ERROR (error)
	RETURN ($$TRUE)
END SUB

END FUNCTION

END PROGRAM